"""
Machine Learning module for Convex Regression
=============================================

This module contains the machine learning methods for Convex Regression.
"""

__all__ = [
    'models',
    'constant'
    ]